#!/bin/bash

function get_filetype_name
{
	if [ -f "$1.bz2" ]; then
		filetype="bz2"
	elif [ -f "$1.xz" ]; then
		filetype="xz"
	elif [ -f "$1.gz" ]; then
		filetype="gz"
	fi

	filename="$1.$filetype"
}

function change_import_status
{
	case "$1" in
		"planned")
			psql -U op5lsu -d op5lsu -c "UPDATE tables_registry SET import_status='planned' WHERE import_status='started';" >/dev/null
			echo "Updating status to: $1"
			;;
		"reset")
			psql -U op5lsu -d op5lsu -c "UPDATE tables_registry SET import_status='planned' WHERE import_status='started';" >/dev/null
			echo "Updating status to: $1"
			;;
		"failed")
			psql -U op5lsu -d op5lsu -c "SELECT change_import_status('messages_$2_$3_$4_$5', 'failed');"
			echo "Updating status to: $1 for messages_$2_$3_$4_$5"
			;;
		"finished")
			psql -U op5lsu -d op5lsu -c "SELECT change_import_status('messages_$2_$3_$4_$5', 'finished');"
			echo "Updating status to: $1 for messages_$2_$3_$4_$5"
			;;
		"started")
			psql -U op5lsu -d op5lsu -c "SELECT change_import_status('messages_$2_$3_$4_$5', 'started');"
			echo "Updating status to: $1 for messages_$2_$3_$4_$5"
			;;
		*)
			echo "Unknown"
			;;
	esac
}

function import
{
	paths="$(psql -U op5lsu op5lsu -t -c "SELECT local_storage_path, remote_storage_path from settings;")"
	localpath=$(echo "$paths" | cut -d ' ' -f 2)
	remotepath=$(echo "$paths" | cut -d ' ' -f 4)
	tmpfilename="$localpath/cfs.$1_$2_$3_$4_00"

	to_time=$((1$4+1-100))

	archive=$(psql -U op5lsu op5lsu -t -c "SELECT get_location(location) FROM tables_registry WHERE name = 'messages_$1_$2_$3_$4'" | cut -d ' ' -f 2);
	if [ $archive = "local" ]; then
		get_filetype_name "$localpath/$1_$2/$3/$4"
	elif [ $archive = "remote" ]; then
		get_filetype_name "$remotepath/$1_$2/$3/$4"
	else # fatal error
		change_import_status "failed" $1 $2 $3 $4
		return 1
	fi
	case "$filetype" in
		"bz2")
			rpm -q pbzip2 > /dev/null
			if [ $? -eq 0 ]; then
				import_cmd="pbzcat"
			else
				import_cmd="bzcat"
			fi
			;;
		"xz")
			import_cmd="xzcat"
			;;
		"gz")
			rpm -q pigz > /dev/null
			if [ $? -eq 0 ]; then
				import_cmd="unpigz -c"
			else
				import_cmd="zcat"
			fi
			;;
		*)
			# Unknown fileformat, aborting
			change_import_status "failed" $1 $2 $3 $4
			return 1
	esac

	if [ ! -f $filename ]; then
		change_import_status "failed" $1 $2 $3 $4
		return 1
	fi

	verify_disk_before_import $filename "$1-$2-$3 $4:00"
	echo "Exit: $?"

	if [ $stop_import -gt 0 ]; then
		change_import_status "failed" $1 $2 $3 $4
		return 1
	fi

	change_import_status "started" $1 $2 $2 $4
	psql -U op5lsu -d op5lsu -f - -1 -v ON_ERROR_STOP=ON <<-SQL
	    CREATE TABLE messages_$1_$2_$3_$4 (LIKE messages INCLUDING DEFAULTS);
	SQL
	if [ $? -gt 0 ]; then
		change_import_status "failed" $1 $2 $3 $4
		return 1
	fi

	nice -n 19 ionice -c3 $import_cmd $filename | psql -U op5lsu -d op5lsu -c "COPY messages_$1_$2_$3_$4 (ip, rtime, facsev, mtime, host, ident, pid, event, msg, fts) FROM STDIN DELIMITER '' NULL AS 'NULL' CSV QUOTE '';"
	if [ $? -gt 0 ]; then
		change_import_status "failed" $1 $2 $3 $4
		psql -U op5lsu -d op5lsu -c "DROP TABLE messages_$1_$2_$3_$4;"
		return 1
	fi

	psql -U op5lsu -d op5lsu -f - -1 -v ON_ERROR_STOP=ON <<-SQL
		ALTER TABLE messages_$1_$2_$3_$4 ADD PRIMARY KEY (id);
		SELECT register_table( 'messages_$1_$2_$3_$4', get_location_mask( 'database' ));
	SQL
		change_import_status "finished" $1 $2 $3 $4
	if [ $? -gt 0 ]; then
		#echo "Error when setting the constraints and indexes on the messages_$1_$2_$3_$4" >> /tmp/some-log-file.log
		change_import_status "failed" $1 $2 $3 $4
		psql -U op5lsu -d op5lsu -c "DROP TABLE messages_$1_$2_$3_$4;"
		return 1
	fi

	if [ $(psql -U op5lsu op5lsu -t -c "SELECT create_fts_index FROM settings WHERE create_fts_index = TRUE") ]; then
		psql -U op5lsu -d op5lsu -f - -1 -v ON_ERROR_STOP=ON <<-SQL
			CREATE INDEX messages_$1_$2_$3_$4_key ON messages_$1_$2_$3_$4 USING gin( to_tsvector('english', fts) );
		SQL
	fi
}

function verify_disk_before_import {
	compressed_filesize_modifier=$(psql -U op5lsu op5lsu -t -c "SELECT compressed_filesize_modifier FROM settings;") #magic number based on research from institutes in switzerland.
	min_disk_size_free=$(psql -U op5lsu op5lsu -t -c "SELECT min_disk_size_free FROM settings;") #in Mb

	my_space_left=$(df -P $PGDATA |awk '{ print $4 }'|grep -v Available)

	min_disk_size_free=$[$min_disk_size_free * 1024]
	filesize=$(ls -l $1| awk '{ print $5 }')
	uncompressed_in_db=$[$filesize * $compressed_filesize_modifier]
	total_size_on_disk=$[$uncompressed_in_db + $min_disk_size_free]

	if [ $total_size_on_disk -gt $my_space_left ]; then
		alert "Aborting Import of $2, not enough space left on device ("$[$total_size_on_disk/1024]"MB estimated/"$[$my_space_left/1024]"MB free)"
		stop_import=1
		return 1
	fi
	return 0
}

function alert {
	logger -s -p daemon.error -t "logserver [IMPORT]" "$1"
}

PID=$$

PGDATA=$(grep "PGDATA=" /etc/init.d/postgresql|cut -d"=" -f 2)

if [ -f "/var/run/import-daemon.pid" -a "$(ps -p $(cat /var/run/import-daemon.pid) -o comm= | grep import-daemon)" ]; then
    echo "another import is running"
    exit
else
    change_import_status "reset"
    echo -n $PID > /var/run/import-daemon.pid
fi

stop_import=0
planned=$(psql -U op5lsu op5lsu -t -c "SELECT name FROM tables_registry WHERE import_status = 'planned' ORDER BY name");
if [ -n "$planned" ]; then
	for import_day in $planned; do
		year=$(echo -n $import_day | cut -d '_' -f 2)
		month=$(echo -n $import_day | cut -d '_' -f 3)
		day=$(echo -n $import_day | cut -d '_' -f 4)
		hour=$(echo -n $import_day | cut -d '_' -f 5)

		if [ $stop_import -eq 0 ]; then
			import $year $month $day $hour
		else
			alert "Recursive aborting import of $year-$month-$day $hour:00"
			change_import_status "failed" $year $month $day $hour
		fi
	done
	sleep 2
fi
